"""
OpenTMI Testcase module
"""
# Internal imports
from opentmi_client.utils.Base import BaseApi
from opentmi_client.utils.decorators import setter_rules
from opentmi_client.api.testcase.OtherInfo import OtherInfo
from opentmi_client.api.testcase.Execution import Execution
from opentmi_client.api.testcase.Status import Status
from opentmi_client.api.testcase.Compatible import Compatible


class Testcase(BaseApi):
    """
    Testcase class
    """
    def __init__(self, tcid):
        """
        Constructor for Testcase
        :param tcid: String
        """
        super().__init__()
        self.tcid = tcid
        self.status = Status()
        self.execution = Execution()
        self.other_info = OtherInfo()
        self.compatible = Compatible()

    def __str__(self):
        """
        Stringify function
        :return: String
        """
        return "{}".format(self.get("tcid", "?"))

    @property
    def tcid(self):
        """
        Getter for test case ID
        :return: String
        """
        return self.get("tcid")

    @tcid.setter
    @setter_rules()
    def tcid(self, value: str):
        """
        Setter for test case ID
        :param value: String
        """
        self.set("tcid", value)

    @property
    def other_info(self):
        """
        Getter for execution
        :return: OtherInfo
        """
        return self.get("other_info")

    @other_info.setter
    @setter_rules(OtherInfo)
    def other_info(self, value: OtherInfo):
        """
        Setter for other_info
        :param value: OtherInfo
        """
        self.set("other_info", value)

    @property
    def execution(self):
        """
        Getter for execution
        :return: String
        """
        return self.get("execution")

    @execution.setter
    @setter_rules(Execution)
    def execution(self, value: Execution):
        """
        Setter for execution
        :param value: String
        """
        self.set("execution", value)

    @property
    def status(self):
        """
        Getter for status
        :return: Status
        """
        return self.get("status")

    @status.setter
    @setter_rules(Status)
    def status(self, value: Status):
        """
        Setter for status
        :param value: Status
        """
        self.set("status", value)

    @property
    def compatible(self):
        """
        Getter for compatible
        :return: Compatible
        """
        return self.get("compatible")

    @compatible.setter
    @setter_rules(Compatible)
    def compatible(self, value: Compatible):
        """
        Setter for compatible
        :param value: Compatible
        """
        self.set("compatible", value)



from os import path
from opentmi_client import Result

dir_path = path.dirname(path.realpath(__file__))
junit_file = path.join(dir_path, "../test/data/junit_simple.xml")
results = Result.from_junit_file(junit_file)

for result in results: print(result)



import unittest
from opentmi_client.api.resource.Resource import Resource
from opentmi_client.api.resource.Status import Status
from opentmi_client.api.resource.Item import Item
from opentmi_client.api.resource.Hardware import Hardware


class TestResource(unittest.TestCase):
    def test_construct(self):
        resource = Resource()
        self.assertIsInstance(resource, Resource)
        self.assertEqual(resource.data, {})

    def test_str(self):
        res = Resource(name="abc")
        self.assertEqual(str(res), "abc")

    def test_properties(self):
        res = Resource(name="abc")
        self.assertEqual(res.name, "abc")
        res.type = 'dut'
        self.assertEqual(res.type, 'dut')
        self.assertIsInstance(res.item, Item)
        self.assertIsInstance(res.status, Status)
        self.assertIsInstance(res.hardware, Hardware)
        res.item.model = 'aa'
        self.assertEqual(res.item.model, 'aa')
        res.status.value = 'active'
        self.assertEqual(res.status.value, 'active')

        res.status.availability = 'free'
        self.assertEqual(res.status.availability, 'free')

        res.status.note = 'node'
        self.assertEqual(res.status.note, 'node')

        res.hardware.hwid = '123'
        self.assertEqual(res.hardware.hwid, '123')


        res.hardware.serial_number = '12'
        self.assertEqual(res.hardware.serial_number, '12')

        self.assertDictEqual(res.data, {
            'name': 'abc',
            'type': 'dut',
            'item': {
                'model': 'aa'
            },
            'status': {
                'value': 'active',
                'note': 'node',
                'availability': 'free'
            },
            'hw': {
                'hwid': '123',
                'sn': '12'
            }
        })



"""
OpenTMI module for Metadata details
"""
from opentmi_client.utils.Base import BaseApi


class Metadata(BaseApi):
    """
    Metadata class
    """

    def append(self, key: str, value: str):
        """
        Setter for test case keywords
        :param key: String
        :param value: String
        """
        self.set(key, value)



# pylint: disable=C0103
"""
OpenTMI Provider module
"""
from opentmi_client.utils.Base import BaseApi
from opentmi_client.utils.decorators import setter_rules


class Provider(BaseApi):
    """
    Provider class. Holds details about who provides DUT's
    """
    def __init__(self, name=None, id_=None, ver=None):
        """
        Constructor for Provider
        :param name: String
        :param id:  String
        :param ver:  String
        """
        super(Provider, self).__init__()
        if name:
            self.name = name
        if id_:
            self.id = id_
        if ver:
            self.ver = ver

    @property
    def name(self):
        """
        Getter for provider name
        :return: String
        """
        return self.get("name")

    @name.setter
    @setter_rules()
    def name(self, value):
        """
        Setter for provider name
        :param value: String
        """
        self.set("name", value)

    @property
    def id(self):
        """
        Getter for provider id
        :return: String
        """
        return self.get("id")

    @id.setter
    @setter_rules()
    def id(self, value):
        """
        Setter for provider id
        :param value: String
        """
        self.set("id", value)

    @property
    def ver(self):
        """
        Getter for provider version
        :return: String
        """
        return self.get("ver")

    @ver.setter
    @setter_rules()
    def ver(self, value):
        """
        Setter for provider version
        :param value: String
        """
        self.set("ver", value)



"""
Collect all public opentmi API's
"""
from opentmi_client.api.client import create, OpenTmiClient
from opentmi_client.api.build import Build, Vcs, Ci, Target, Hardware
from opentmi_client.api.result import Result
from opentmi_client.api.result import Job
from opentmi_client.api.result import Environment
from opentmi_client.api.result import Sut
from opentmi_client.api.result import Dut
from opentmi_client.api.result import Execution
from opentmi_client.api.result import Framework
from opentmi_client.api.result import File
from opentmi_client.api.result import Provider
from opentmi_client.api.event import Event
from opentmi_client.api.testcase import Testcase

Client = OpenTmiClient



"""
OpenTMI module for Resource
"""
from opentmi_client.utils.Base import BaseApi
from opentmi_client.utils.decorators import setter_rules
from opentmi_client.api.resource.Item import Item
from opentmi_client.api.resource.Status import Status
from opentmi_client.api.resource.Hardware import Hardware


class Resource(BaseApi):
    """
    Resource Class
    """

    def __init__(self, name=""):
        """
        Constructor for Resource
        """
        super().__init__()
        self.name = name
        self.item = Item()
        self.status = Status()
        self.hardware = Hardware()

    def __str__(self):
        return self.name

    @property
    def name(self):
        """
        Getter for name
        :return: String
        """
        return self.get("name")

    @name.setter
    @setter_rules()
    def name(self, value):
        """
        Setter for name
        :param value: string
        """
        self.set("name", value)

    @property
    def type(self):
        """
        Getter for type
        :return: String
        """
        return self.get("type")

    @type.setter
    @setter_rules(enum=[
        'system',
        'dut',
        'instrument',
        'accessories',
        'computer',
        'room'
    ])
    def type(self, value):
        """
        Setter for type
        :param value: string
        """
        self.set("type", value)

    @property
    def item(self):
        """
        Getter for item
        :return: Item
        """
        return self.get("item")

    @item.setter
    @setter_rules(Item)
    def item(self, value):
        """
        Setter for item
        :param value: Item
        """
        self.set("item", value)

    @property
    def status(self) -> Status:
        """
        Getter for status
        :return: Status
        """
        return self.get("status")

    @status.setter
    @setter_rules(Status)
    def status(self, value):
        """
        Setter for status
        :param value: Status
        """
        self.set("status", value)

    @property
    def hardware(self) -> Hardware:
        """
        Getter for hardware
        :return: Hardware
        """
        return self.get("hw")

    @hardware.setter
    @setter_rules(Hardware)
    def hardware(self, value):
        """
        Setter for hardware
        :param value: Hardware
        """
        self.set("hw", value)



from opentmi_client import OpenTmiClient, Event


client = OpenTmiClient(port=3000)
client.login("admin", "admin")

# create event and send it
event = Event()
event.priority.level = "info"
event.priority.facility = "result"
event.ref.result = "5697740f956cd2fd35c69062"
event.traceid = "123"
print(event.data)
client.post_event(event)



import unittest
from opentmi_client.api import Event


class TestEvent(unittest.TestCase):
    def test_construct(self):
        event = Event()
        self.assertIsInstance(event, Event)
        self.assertEqual(event.data, {})

    def test_str(self):
        event = Event()
        event.eid = "iid"
        self.assertEqual(str(event), "iid")

    def test_properties(self):
        event = Event()
        event.msgid = 'ALLOCATED'
        self.assertEqual(event.msgid, 'ALLOCATED')
        event.traceid = "asd"
        self.assertEqual(event.traceid, "asd")
        event.msg = "abc"
        self.assertEqual(event.msg, "abc")
        event.tag = "taag"
        self.assertEqual(event.tag, "taag")
        event.duration = 123.0
        self.assertEqual(event.duration, 123.0)
        event.eid = "aid"
        self.assertEqual(event.eid, "aid")
        spare = {"a": 123}
        event.spare = spare
        self.assertEqual(event.spare, spare)
        self.assertDictEqual(event.data, {
            "msgid": "ALLOCATED",
            "traceid": "asd",
            "msg": "abc",
            "tag": "taag",
            "duration": 123.0,
            "id": "aid",
            "spare": spare,
        })

    def test_priority(self):
        event = Event()
        event.priority.facility = "testcase"
        self.assertEqual(event.priority.facility, "testcase")
        event.priority.level = "warning"
        self.assertEqual(event.priority.level, "warning")
        self.assertDictEqual(event.data, {
            "priority": {
                "facility": "testcase",
                "level": "warning"
            }
        })

    def test_ref(self):
        event = Event()
        event.ref.resource = "123"
        self.assertEqual(event.ref.resource, "123")
        event.ref.testcase = "tcs"
        self.assertEqual(event.ref.testcase, "tcs")
        event.ref.result = "res"
        self.assertEqual(event.ref.result, "res")
        self.assertDictEqual(event.data, {
            "ref": {
                "resource": "123",
                "testcase": "tcs",
                "result": "res"
            }
        })



#!/usr/bin/env python

"""
Command line interface module for OpenTMI client
License: MIT
"""

from __future__ import print_function
import sys
import json
import argparse
import logging
import pkg_resources  # part of setuptools
from opentmi_client.api import OpenTmiClient
from opentmi_client.utils.exceptions import OpentmiException

EXIT_CODE_SUCCESS = 0
EXIT_CODE_NOT_IMPLEMENTED = 1
EXIT_CODE_CONNECTION_ERROR = 60
EXIT_CODE_OPERATION_TIMEOUT = 61
EXIT_CODE_INVALID_PARAMETERS = 62
EXIT_CODE_OPERATION_FAILED = 63


def get_subparser(subparsers, name, func=None, **kwargs):
    """
    Get subparser
    :param subparsers:
    :param name:
    :param func:
    :param kwargs:
    :return: Parser
    """
    tmp_parser = subparsers.add_parser(name, **kwargs)
    if func:
        tmp_parser.set_defaults(func=func)
    return tmp_parser


class OpentTMIClientCLI(object):
    """
    OpenTMICLientCLI
    """

    def __init__(self, args=None):
        """
        Constructor for CLI
        :param args:
        """
        self.console_handler = logging.StreamHandler()
        self.logger = logging.getLogger("opentmi")
        self.logger.handlers = [self.console_handler]
        if args is None:
            args = sys.argv[1:]
        self.args = self.argparser_setup(args)
        self.set_log_level_from_verbose()

    def execute(self):
        """
        Execute
        :return: 0
        """
        if hasattr(self.args, "func") and callable(self.args.func):
            try:
                return self.args.func(self.args)
            except NotImplementedError as error:
                self.logger.error("Not implemented %s", str(error))
                return EXIT_CODE_NOT_IMPLEMENTED
        self.parser.print_usage()
        return EXIT_CODE_SUCCESS

    def argparser_setup(self, sysargs):
        """
        Configure CLI (Command Line Options) options
        :param self:
        :param sysargs:
        :return: Returns OptionParser's tuple of (options, arguments)
        """
        parser = argparse.ArgumentParser()

        parser.add_argument('-v',
                            dest="verbose",
                            action="count",
                            help="verbose level... repeat up to three times.")

        parser.add_argument('-s', '--silent',
                            dest="silent", default=False,
                            action="store_true",
                            help="Silent - only errors will be printed")

        parser.add_argument('--host',
                            dest='host',
                            default='localhost',
                            help='OpenTMI host, default: localhost')

        parser.add_argument('--user',
                            dest='user',
                            default=None,
                            help='username')

        parser.add_argument('--password',
                            dest='password',
                            default=None,
                            help='password')

        parser.add_argument('--token',
                            dest='token',
                            default=None,
                            help='Authentication token')

        parser.add_argument('--token_service',
                            dest='token_service',
                            default=None,
                            help='Optional authentication service')

        parser.add_argument('-p', '--port',
                            dest='port',
                            type=int,
                            default=0,
                            help='OpenTMI port')

        subparsers = parser.add_subparsers(title='subcommand',
                                           help='sub-command help',
                                           metavar='<subcommand>')
        get_subparser(subparsers, 'version',
                      func=self.subcmd_version_handler,
                      help='Display version information')

        parser_list = get_subparser(subparsers, 'list',
                                    func=self.subcmd_list_handler,
                                    help='List something')

        parser_list.add_argument('--json',
                                 dest='json',
                                 default=False,
                                 action='store_true',
                                 help='results as json')

        parser_list.add_argument('--testcases',
                                 dest='testcases',
                                 action='store_true',
                                 default=None,
                                 help='Testcases')

        parser_list.add_argument('--campaigns',
                                 dest='campaigns',
                                 action='store_true',
                                 default=None,
                                 help='Campaigns')

        parser_list.add_argument('--builds',
                                 dest='builds',
                                 action='store_true',
                                 default=None,
                                 help='Builds')

        parser_store = get_subparser(subparsers, 'store',
                                     help='Create something')

        subsubparsers = parser_store.add_subparsers(title='subcommand',
                                                    help='sub-command help',
                                                    metavar='<subcommand>')

        parser_store_testcase = get_subparser(subsubparsers, 'testcase',
                                              func=self.subcmd_store_testcase,
                                              help='Store Testcase')
        parser_store_testcase.add_argument('--file',
                                           dest='file',
                                           default=None,
                                           help='Filename',
                                           type=self.read_json_file,
                                           required=True)

        parser_store_result = get_subparser(subsubparsers, 'result',
                                            func=self.subcmd_store_result,
                                            help='Store Test Result')
        parser_store_result.add_argument('--file',
                                         dest='file',
                                         default=None,
                                         help='Filename',
                                         type=self.read_json_file,
                                         required=True)

        parser_store_build = get_subparser(subsubparsers, 'build',
                                           func=self.subcmd_store_build,
                                           help='Store Build')
        parser_store_build.add_argument('--file',
                                        dest='file',
                                        default=None,
                                        help='Filename',
                                        type=self.read_json_file,
                                        required=True)

        args = parser.parse_args(args=sysargs)
        self.parser = parser
        return args

    def read_json_file(self, filename):
        """
        :param filename: json filename to be read
        :returns: Dict
        """
        try:
            with open(filename) as data_file:
                return json.load(data_file)
        except IOError as error:
            self.logger.error("Given file (%s) is not valid! %s", filename, error)
            raise argparse.ArgumentTypeError(error)

    def set_log_level_from_verbose(self):
        """
        Sets logging level, silent, or some of verbose level
        Args:
             command line arguments
        """
        if self.args.silent or not self.args.verbose:
            self.console_handler.setLevel('ERROR')
            self.logger.setLevel('ERROR')
        elif self.args.verbose == 1:
            self.console_handler.setLevel('WARNING')
            self.logger.setLevel('WARNING')
        elif self.args.verbose == 2:
            self.console_handler.setLevel('INFO')
            self.logger.setLevel('INFO')
        elif self.args.verbose >= 3:
            self.console_handler.setLevel('DEBUG')
            self.logger.setLevel('DEBUG')

    def subcmd_version_handler(self, _args):
        """
        :param self:
        :param _args:
        :return:
        """
        versions = pkg_resources.require("opentmi_client")
        if self.args.verbose:
            for ver in versions:
                print(ver)
        else:
            print(versions[0].version)
        return EXIT_CODE_SUCCESS

    def subcmd_store_handler(self, _args):
        """
        :param self:
        :param _args:
        :return:
        """

        raise NotImplementedError('store')

    @staticmethod
    def create_client(args):
        """
        Create OpenTmiClient instance based on args
        :param args: arguments
        :return: OpenTmiClient instance
        """
        client = OpenTmiClient(host=args.host, port=args.port)
        if args.user:
            if args.password:
                client.login(args.user, args.password)
            else:
                raise OpentmiException("password missing")
        elif args.token:
            service = args.token_service or "github"
            client.login_with_access_token(args.token, service)
        return client

    def subcmd_store_build(self, args):
        """
        :param self:
        :param args:
        :return:
        """
        client = self.create_client(args)
        client.upload_build(args.file)
        return EXIT_CODE_SUCCESS

    def subcmd_store_testcase(self, args):
        """
        :param self:
        :param args:
        :return:
        """
        client = self.create_client(args)
        client.update_testcase(args.file)
        return EXIT_CODE_SUCCESS

    def subcmd_store_result(self, args):
        """
        :param self:
        :param args:
        :return:
        """
        client = self.create_client(args)
        client.upload_results(args.file)
        return EXIT_CODE_SUCCESS

    def subcmd_list_handler(self, args):
        """
        :param args:
        :return:
        """
        client = self.create_client(args)
        if args.testcases:
            testcases = client.get_testcases()
            if args.json:
                print(json.dumps(testcases))
            else:
                print("Test cases:")
                for test_case in testcases:
                    print(test_case['tcid'])
        elif args.campaigns:
            campaigns = client.get_campaign_names()
            if args.json:
                print(campaigns)
            else:
                for campaign in campaigns:
                    print(campaign)
        return 0


def opentmiclient_main():
    """
    Function used to drive CLI (command line interface) application.
    Function exits back to command line with ERRORLEVEL
    Returns:
        Function exits with success-code
    """
    cli = OpentTMIClientCLI()
    try:
        sys.exit(cli.execute())
    except OpentmiException as error:
        print(str(error))
        sys.exit(EXIT_CODE_OPERATION_FAILED)


if __name__ == '__main__':
    opentmiclient_main()



"""
OpentTMI module for Priority
"""
from opentmi_client.utils.Base import BaseApi
from opentmi_client.utils.decorators import setter_rules


class Priority(BaseApi):
    """
    Priority class
    """

    @property
    def level(self):
        """
        Getter for level
        :return: String
        """
        return self.get("level")

    @level.setter
    @setter_rules(
        enum=['emerg', 'alert', 'crit', 'err',
              'warning', 'notice', 'info', 'debug'])
    def level(self, value):
        """
        Setter for level
        :param value: string
        :return: Priority
        """
        return self.set("level", value)

    @property
    def facility(self):
        """
        Getter for facility
        :return: String
        """
        return self.get("facility")

    @facility.setter
    @setter_rules(
        enum=['auth', 'cron', 'mail', 'news', 'syslog',
              'user', 'result', 'resource', 'testcase'])
    def facility(self, value):
        """
        Setter for facility
        :param value: string
        :return: Priority
        """
        return self.set("facility", value)



"""
OpenTMI module for Environment details
"""
from opentmi_client.utils.Base import BaseApi
from opentmi_client.utils.decorators import setter_rules
from opentmi_client.api.result.Framework import Framework

class Environment(BaseApi):
    """
    Environment class
    """
    def __init__(self, framework=None):
        """
        constructor for Environment
        :param framework: String
        """
        super(Environment, self).__init__()
        self.framework = framework if framework else Framework()

    @property
    def ref(self):
        """
        Getter for environment reference
        :return: String
        """
        return self.get("ref")

    @ref.setter
    @setter_rules()
    def ref(self, value):
        self.set("ref", value)

    @property
    def framework(self):
        """
        Getter for framework
        :return: Framework
        """
        return self.get("framework")

    @framework.setter
    @setter_rules(value_type=Framework)
    def framework(self, value):
        self.set("framework", value)



"""
OpentTMI module for DUT (Device under test)
"""
from opentmi_client.utils.Base import BaseApi
from opentmi_client.utils.decorators import setter_rules
from opentmi_client.api.result.Provider import Provider


class Dut(BaseApi):
    """
    Dut class
    """

    @property
    def count(self):
        """
        Getter for dut count
        :return: Integer
        """
        return self.get("count")

    @count.setter
    @setter_rules(value_type=int)
    def count(self, value):
        """
        Setter for dut count
        :param value: integer
        :return: value
        """
        return self.set("count", value)

    @property
    def type(self):
        """
        Getter for DUT type
        :return: String
        """
        return self.get("type")

    @type.setter
    @setter_rules(enum="hw simulator process")
    def type(self, value):
        """
        Setter for DUT type
        :param value: String, hw|simulator|process
        :return: value
        """
        return self.set("type", value)

    @property
    def ref(self):
        """
        Getter for reference
        :return: String
        """
        return self.get("ref")

    @ref.setter
    @setter_rules()
    def ref(self, value):
        """
        Setter for reference
        :param value:
        :return:
        """
        return self.set("ref", value)

    @property
    def vendor(self):
        """
        Getter for vendor
        :return: String
        """
        return self.get("vendor")

    @vendor.setter
    @setter_rules()
    def vendor(self, value):
        """
        Setter for DUT vendor
        :param value: String
        :return: value
        """
        return self.set("vendor", value)

    @property
    def model(self):
        """
        Getter for dut model
        :return: String
        """
        return self.get("model")

    @model.setter
    @setter_rules()
    def model(self, value):
        """
        Setter for dut model
        :param value: String
        :return: value
        """
        return self.set("model", value)

    @property
    def ver(self):
        """
        Getter for dut version
        :return: String
        """
        return self.get("ver")

    @ver.setter
    @setter_rules()
    def ver(self, value):
        """
        Setter for version
        :param value: String
        :return: value
        """
        return self.set("ver", value)

    @property
    def serial_number(self):
        """
        Getter for dut Serial Number
        :return: String
        """
        return self.get("sn")

    @serial_number.setter
    @setter_rules()
    def serial_number(self, value):
        """
        Setter for Serial Number
        :param value: String
        :return: value
        """
        return self.set("sn", value)

    @property
    def provider(self):
        """
        Getter for dut provider
        :return: String
        """
        return self.get("provider")

    @provider.setter
    @setter_rules(value_type=Provider)
    def provider(self, value):
        """
        Setter for dut provider
        :return: Provider
        """
        return self.set("provider", value)



""" OpenTMI module """
# Internal imports
from opentmi_client.utils.Base import BaseApi
from opentmi_client.utils.decorators import setter_rules


class Verification(BaseApi):
    """ Execution class """

    @property
    def value(self):
        """
        Getter for value
        :return: bool
        """
        return self.get("value")

    @value.setter
    @setter_rules(bool)
    def value(self, value: bool):
        """
        Setter for test case verification value
        :param value: bool
        """
        self.set("value", value)



import unittest
from opentmi_client.api import Sut


class TestResultSut(unittest.TestCase):
    def test_construct(self):
        sut = Sut()
        sut.branch = 'test'
        self.assertIsInstance(sut, Sut)
        self.assertEqual(sut.branch, 'test')
        self.assertEqual(sut.data, {'branch': 'test'})

    def test_build_sha1(self):
        sut = Sut()
        sut.build_sha1 = '123'
        self.assertEqual(sut.build_sha1, '123')
        self.assertEqual(sut.data, {'buildSha1': '123'})

    def test_commit_id(self):
        sut = Sut()
        sut.commit_id = '123'
        self.assertEqual(sut.commit_id, '123')
        self.assertEqual(sut.data, {'commitId': '123'})

    def test_tag(self):
        sut = Sut()
        sut.tag = ['123', '456']
        self.assertEqual(sut.tag, ['123', '456'])
        self.assertEqual(sut.data, {'tag': ['123', '456']})

    def test_git_url(self):
        sut = Sut()
        sut.git_url = '123'
        self.assertEqual(sut.git_url, '123')
        self.assertEqual(sut.data, {'gitUrl': '123'})

    def test_fut(self):
        sut = Sut()
        sut.append_fut('123')
        self.assertEqual(sut.fut, ['123'])
        self.assertEqual(sut.data, {'fut': ['123']})

    def test_cut(self):
        sut = Sut()
        sut.append_cut('123')
        self.assertEqual(sut.cut, ['123'])
        self.assertEqual(sut.data, {'cut': ['123']})

    def test_ref(self):
        sut = Sut()
        sut.ref = '123'
        self.assertEqual(sut.ref, '123')
        self.assertEqual(sut.data, {'ref': '123'})



""" OpenTMI OtherInfo module """
# Internal imports
from opentmi_client.utils.Base import BaseApi
from opentmi_client.utils.decorators import setter_rules


class OtherInfo(BaseApi):
    """ OtherInfo class """

    @property
    def title(self):
        """
        Getter for title
        :return: String
        """
        return self.get("title")

    @title.setter
    @setter_rules()
    def title(self, value: str):
        """
        Setter for test case title
        :param value: String
        """
        self.set("title", value)

    @property
    def type(self):
        """
        Getter for type
        :return: String
        """
        return self.get("type")

    @type.setter
    @setter_rules(enum=[
        'installation',
        'compatibility',
        'smoke',
        'regression',
        'acceptance',
        'alpha',
        'beta',
        'stability',
        'functional',
        'destructive',
        'performance',
        'reliability'])
    def type(self, value: str):
        """
        Setter for test case type
        :param value: String
        """
        self.set("type", value)

    @property
    def purpose(self):
        """
        Getter for purpose
        :return: String
        """
        return self.get("purpose")

    @purpose.setter
    @setter_rules()
    def purpose(self, value: str):
        """
        Setter for test case purpose
        :param value: String
        """
        self.set("purpose", value)

    @property
    def description(self):
        """
        Getter for description
        :return: String
        """
        return self.get("description")

    @description.setter
    @setter_rules()
    def description(self, value: str):
        """
        Setter for test case description
        :param value: String
        """
        self.set("description", value)

    @property
    def layer(self):
        """
        Getter for layer
        :return: String
        """
        return self.get("layer")

    @layer.setter
    @setter_rules(enum=['L1', 'L2', 'L3', 'unknown'])
    def layer(self, value: str):
        """
        Setter for test case layer
        :param value: String
        """
        self.set("layer", value)

    @property
    def components(self):
        """
        Getter for components
        :return: list<str>
        """
        return self.get("components")

    @components.setter
    @setter_rules(value_type=list, each_type=str)
    def components(self, value: list):
        """
        Setter for test case components
        :param value: String
        """
        self.set("components", value)

    @property
    def features(self):
        """
        Getter for features
        :return: list<str>
        """
        return self.get("features")

    @features.setter
    @setter_rules(value_type=list, each_type=str)
    def features(self, value: list):
        """
        Setter for test case features
        :param value: String
        """
        self.set("features", value)

    @property
    def keywords(self):
        """
        Getter for keywords
        :return: list<str>
        """
        return self.get("keywords")

    @keywords.setter
    @setter_rules(value_type=list, each_type=str)
    def keywords(self, value: list):
        """
        Setter for test case keywords
        :param value: String
        """
        self.set("keywords", value)



""" OpenTMI module """
# Internal imports
from opentmi_client.utils.Base import BaseApi
from opentmi_client.utils.decorators import setter_rules
from opentmi_client.api.testcase.Verification import Verification


class Status(BaseApi):
    """ Status class """
    def __init__(self):
        """ Constructor for Status """
        super().__init__()
        self.verification = Verification()

    @property
    def value(self):
        """
        Getter for value
        :return: String
        """
        return self.get("value")

    @value.setter
    @setter_rules(enum=['unknown', 'released', 'development', 'maintenance', 'broken'])
    def value(self, value: str):
        """
        Setter for test case status
        :param value: String
        """
        self.set("value", value)

    @property
    def verification(self):
        """
        Getter for verification
        :return: Verification
        """
        return self.get("verification")

    @verification.setter
    @setter_rules(value_type=Verification)
    def verification(self, value: Verification):
        """
        Setter for test case verification
        :param value: Verification
        """
        self.set("verification", value)



"""
OpenTMI module for Event
"""
from opentmi_client.utils.Base import BaseApi
from opentmi_client.utils.decorators import setter_rules
from opentmi_client.api.event.Priority import Priority
from opentmi_client.api.event.Ref import Ref


class Event(BaseApi):
    """
    Event Class
    """

    def __init__(self):
        """
        Constructor for Event
        """
        super(Event, self).__init__()
        self.priority = Priority()
        self.ref = Ref()

    def __str__(self):
        return "{}".format(self.get("id", "unknown"))

    @property
    def traceid(self):
        """
        Getter for traceid
        :return: String
        """
        return self.get("traceid")

    @traceid.setter
    @setter_rules()
    def traceid(self, value):
        """
        Setter for traceid
        :param value: string
        """
        self.set("traceid", value)

    @property
    def eid(self):
        """
        Getter for event id
        :return: String
        """
        return self.get("id")

    @eid.setter
    @setter_rules()
    def eid(self, value):
        """
        Setter for event id
        :param value: string
        """
        self.set("id", value)

    @property
    def msgid(self):
        """
        Getter for msgid
        :return: String
        """
        return self.get("msgid")

    @msgid.setter
    @setter_rules(
        enum=['ALLOCATED', 'RELEASED', 'ENTER_MAINTENANCE',
              'EXIT_MAINTENANCE', 'CREATED', 'DELETED', 'FLASHED'])
    def msgid(self, value):
        """
        Setter for msgid
        :param value: string
        """
        self.set("msgid", value)

    @property
    def msg(self):
        """
        Getter for msg
        :return: String
        """
        return self.get("msg")

    @msg.setter
    @setter_rules()
    def msg(self, value):
        """
        Setter for msg
        :param value: string
        """
        self.set("msg", value)

    @property
    def tag(self):
        """
        Getter for tag
        :return: String
        """
        return self.get("tag")

    @tag.setter
    @setter_rules()
    def tag(self, value):
        """
        Setter for tag
        :param value: string
        """
        self.set("tag", value)

    @property
    def duration(self):
        """
        Getter for duration
        :return: float
        """
        return self.get("duration")

    @duration.setter
    @setter_rules(value_type=float)
    def duration(self, value):
        """
        Setter for duration
        :param value: float
        """
        self.set("duration", value)

    @property
    def spare(self):
        """
        Getter for spare
        :return: String
        """
        return self.get("spare")

    @spare.setter
    def spare(self, value):
        """
        Setter for spare
        :param value: *
        """
        self.set("spare", value)

    @property
    def priority(self):
        """
        Getter for priority
        :return: Priority
        """
        return self.get("priority")

    @priority.setter
    @setter_rules(value_type=Priority)
    def priority(self, value):
        """
        Setter for priority
        :param value: Priority
        """
        self.set("priority", value)

    @property
    def ref(self):
        """
        Getter for ref
        :return: Ref
        """
        return self.get("ref")

    @ref.setter
    @setter_rules(value_type=Ref)
    def ref(self, value):
        """
        Setter for ref
        :param value: Ref
        """
        self.set("ref", value)



"""
OpenTMI module for Event
"""
from opentmi_client.utils.Base import BaseApi
from opentmi_client.utils.decorators import setter_rules


class Ref(BaseApi):
    """
    Ref Class
    """

    @property
    def result(self):
        """
        Getter for result
        :return: String
        """
        return self.get("result")

    @result.setter
    @setter_rules()
    def result(self, value):
        """
        Setter for result
        :param value: string
        :return: Ref
        """
        return self.set("result", value)

    @property
    def testcase(self):
        """
        Getter for testcase
        :return: String
        """
        return self.get("testcase")

    @testcase.setter
    @setter_rules()
    def testcase(self, value):
        """
        Setter for testcase
        :param value: string
        :return: Ref
        """
        return self.set("testcase", value)

    @property
    def resource(self):
        """
        Getter for resource
        :return: String
        """
        return self.get("resource")

    @resource.setter
    @setter_rules()
    def resource(self, value):
        """
        Setter for resource
        :param value: string
        :return: Ref
        """
        return self.set("resource", value)



"""
Collect all utils API's
"""
from opentmi_client.utils.tools import is_object_id
from opentmi_client.utils.tools import resolve_host, resolve_token
from opentmi_client.utils.tools import archive_files
from opentmi_client.utils.tools import remove_empty_from_dict
from opentmi_client.utils.logger import get_logger
from opentmi_client.utils.exceptions import OpentmiException
from opentmi_client.utils.exceptions import TransportException
from opentmi_client.utils.Query import Query, Find, Distinct
from opentmi_client.utils.decorators import setter_rules, requires_logged_in



"""
Generic tools function
"""
import re
import zipfile
import os
import six


def is_object_id(value):
    """
    Check if value is mongodb ObjectId
    :param value:
    :return: Boolean
    """
    if not isinstance(value, six.string_types):
        return False

    objectid_re = r"^[0-9a-fA-F]{24}$"
    match = re.match(objectid_re, value)
    return bool(match)


def resolve_host(host, port=None):
    """
    Resolve host from given arguments
    :param host: string
    :param port: number
    :return:
    """
    if port and port != 80:
        host += ":" + str(port)
    if not host.startswith("http"):
        host = "http://" + host
    return host

def resolve_token(host):
    """
    Resolve access token from host string
    Format: https://<token>@<url>
    :param host: host as a string
    :return: token or None
    """
    host_re = r"^https?://([a-zA-Z0-9\-_]+?\.[a-zA-Z0-9\-_]+\.[a-zA-Z0-9\-_]+)\@.+$"
    match = re.match(host_re, host)
    return match.group(1) if match else None

def archive_files(files, zip_filename, base_path=""):
    """
    Archive given files
    :param files: list of file names
    :param zip_filename: target zip filename
    :param base_path: base path for files
    :return:
    """
    zip_file = zipfile.ZipFile(zip_filename, "w")
    for filename in files:
        zip_file.write(os.path.join(base_path, filename), filename)
    zip_file.close()
    return zip_filename

def remove_empty_from_dict(dictionary):
    """
    Remove all empty items from nested object
    :param dictionary: dict
    :return: dict
    """
    if isinstance(dictionary, dict):
        try:
            return dict((k, remove_empty_from_dict(v)) for k, v in dictionary.iteritems() if
                        v and remove_empty_from_dict(v))
        except AttributeError:
            return dict((k, remove_empty_from_dict(v)) for k, v in dictionary.items() if
                        v and remove_empty_from_dict(v))
    elif isinstance(dictionary, list):
        return [remove_empty_from_dict(v) for v in dictionary if v and remove_empty_from_dict(v)]
    else:
        return dictionary



"""
OpenTMI Data base class
"""

from json import dumps
from pydash import get, set_, unset, map_values_deep
from opentmi_client.utils import remove_empty_from_dict


class BaseApi(object):
    """
    Base class for Data record
    """
    def __init__(self):
        """
        Constructor for BaseApi
        """
        self._data = {}

    @property
    def _id(self):
        """
        Getter for document id
        :return: document _id
        """
        return self.get('_id')

    @_id.setter
    def _id(self, value):
        """
        document id setter
        :param value: _id
        :return: _id
        """
        return self.set('_id', value)

    @property
    def is_empty(self):
        """
        :return: True data is empty
        """
        return len(self._data.keys()) == 0

    def set_data(self, data):
        """
        Set whole data dictionary
        :param data: Dictionary
        :return: None
        """
        self._data = data

    @property
    def data(self):
        """
        Get plain Dictionary object which are suitable for OpenTMI backend
        :return: Dictionary containsi whole data
        """
        data = map_values_deep(self._data, lambda x: x.data if isinstance(x, BaseApi) else x)
        return remove_empty_from_dict(data)

    @data.setter
    def data(self, values):
        """
        Set plain dictionary object which are suitable for OpenTMI backend
        :param values:
        :return:
        """
        data = remove_empty_from_dict(values)

        def has_attribute(ref, key):
            """
            Validates that attribute key has in ref
            :param ref: Object
            :param key: String
            :return: None
            :raise KeyError: if does not exists
            """
            if not hasattr(ref, key):
                raise KeyError("Key '{}' does not exists".format(key))

        def fnc(value, path):
            """
            mapper function
            :param value: Value to be set
            :param path: array of nested path
            :return:
            """
            ref = self
            for key in path[0:-1]:
                has_attribute(ref, key)
                ref = getattr(ref, key)
            key = path[-1]
            has_attribute(ref, key)
            setattr(ref, key, value)
        map_values_deep(data, fnc)

    def get(self, key, default=None):
        """
        Get value based on key
        :param key: String
        :param default: Default value if not found
        :return: Value for key. Some keys presents another BaseApi object
        """
        return get(self._data, key, default)

    def set(self, key, value):
        """
        Set value for key
        :param key: String
        :param value: new value for key
        :return: value
        """
        set_(self._data, key, value)
        return value

    def unset(self, key):
        """
        Remove key from object
        :param key: String
        :return: self
        """
        unset(self._data, key)
        return self

    def __str__(self):
        """
        Returns stringified json object
        :return: String
        """
        return dumps(self.data, indent=2, default=lambda o: o.data)



"""
Build module
"""
from opentmi_client.api.build.Build import Build
from opentmi_client.api.build.Vcs import Vcs
from opentmi_client.api.build.Ci import Ci
from opentmi_client.api.build.Ci import Location
from opentmi_client.api.build.Target import Target
from opentmi_client.api.build.Hardware import Hardware



""" OpenTMI module """
# Internal imports
from opentmi_client.utils.Base import BaseApi
from opentmi_client.utils.decorators import setter_rules
from opentmi_client.api.testcase.Skip import Skip


class Execution(BaseApi):
    """ Execution class """
    def __init__(self):
        """ Constructor for Execution """
        super().__init__()
        self.skip = Skip()

    @property
    def skip(self):
        """
        Getter for skip
        :return: Skip
        """
        return self.get("skip")

    @skip.setter
    @setter_rules(value_type=Skip)
    def skip(self, value: Skip):
        """
        Setter for test case skip
        :param value: Skip
        """
        self.set("skip", value)



"""
OpenTMI module for Target
"""
from opentmi_client.utils.Base import BaseApi
from opentmi_client.utils.decorators import setter_rules


class Hardware(BaseApi):
    """
    Target Class
    """

    @property
    def vendor(self):
        """
        Getter for vendor
        :return: String
        """
        return self.get("vendor")

    @vendor.setter
    @setter_rules()
    def vendor(self, value):
        """
        Setter for vendor
        :param value: String
        :return: value
        """
        return self.set("vendor", value)

    @property
    def model(self):
        """
        Getter for model
        :return: String
        """
        return self.get("model")

    @model.setter
    @setter_rules()
    def model(self, value):
        """
        Setter for model
        :param value: String
        :return: value
        """
        return self.set("model", value)

    @property
    def rev(self):
        """
        Getter for model
        :return: String
        """
        return self.get("rev")

    @rev.setter
    @setter_rules()
    def rev(self, value):
        """
        Setter for rev
        :param value: String
        :return: value
        """
        return self.set("rev", value)

    @property
    def meta(self):
        """
        Getter for meta
        :return: String
        """
        return self.get("meta")

    @meta.setter
    @setter_rules()
    def meta(self, value):
        """
        Setter for rev
        :param value: String
        :return: value
        """
        return self.set("meta", value)



"""
OpenTMI module for Resource.hw
"""
from opentmi_client.utils.Base import BaseApi
from opentmi_client.utils.decorators import setter_rules


class Hardware(BaseApi):
    """
    Hardware Class
    """

    @property
    def serial_number(self):
        """
        Getter for serial_number
        :return: String
        """
        return self.get("sn")

    @serial_number.setter
    @setter_rules()
    def serial_number(self, value):
        """
        Setter for serial_number
        :param value: string
        """
        self.set("sn", value)

    @property
    def hwid(self):
        """
        Getter for hwid
        :return: String
        """
        return self.get("hwid")

    @hwid.setter
    @setter_rules()
    def hwid(self, value):
        """
        Setter for hwid
        :param value: string
        """
        self.set("hwid", value)



from opentmi_client import Result


def reducer_func(result_obj, value, key):
    # This function remap key to proper Result API
    if key == "result":
        # custom json has result -key, but right place is result.verdict
        result_obj.verdict = value.lower() # allowed values are lower level "pass"
    elif key == "test.name":
        # this is test case id ("name")
        result_obj.tcid = value
    return result_obj


custom_json = {
    "result": "PASS",
    "test": {
        "name": "hello"
    }
}
result = Result.from_dict(custom_json, reducer=reducer_func)
print(result)



"""
OpenTMI module for Resource.Item
"""
from opentmi_client.utils.Base import BaseApi
from opentmi_client.utils.decorators import setter_rules


class Item(BaseApi):
    """
    Item Class
    """

    @property
    def model(self):
        """
        Getter for model
        :return: String
        """
        return self.get("model")

    @model.setter
    @setter_rules()
    def model(self, value):
        """
        Setter for model
        :param value: string
        """
        self.set("model", value)



"""
Testcase module
"""
from opentmi_client.api.testcase.Testcase import Testcase
from opentmi_client.api.testcase.Execution import Execution
from opentmi_client.api.testcase.OtherInfo import OtherInfo
from opentmi_client.api.testcase.Status import Status
from opentmi_client.api.testcase.Skip import Skip
from opentmi_client.api.testcase.Verification import Verification
from opentmi_client.api.testcase.Compatible import Compatible



import unittest
from opentmi_client.api import Build, Ci, Vcs, Target, Hardware


class TestBuild(unittest.TestCase):
    def test_construct(self):
        build = Build()
        build.name = 'test'
        self.assertIsInstance(build, Build)
        self.assertEqual(build.name, 'test')
        self.assertEqual(build.data, {'name': 'test'})

    def test_type(self):
        build = Build()
        with self.assertRaises(ValueError):
            build.type = 'invalid'
        build.type = 'test'
        self.assertEqual(build.type, 'test')

    def test_compiledBy(self):
        build = Build()
        with self.assertRaises(ValueError):
            build.compiled_by = 'invalid'
        build.compiled_by = 'CI'
        self.assertEqual(build.compiled_by, 'CI')

    def test_ci(self):
        build = Build()
        with self.assertRaises(TypeError):
            build.ci_tool = 'invalid'
        ci = Ci()
        with self.assertRaises(ValueError):
            ci.system = 'invalid'
        build.ci_tool = ci
        build.ci_tool.system = 'Jenkins'
        self.assertEqual(ci.system, 'Jenkins')


    def test_vcs(self):
        build = Build()
        with self.assertRaises(TypeError):
            build.vcs = 'invalid'
        vcs = Vcs()
        build.vcs = [vcs]
        build.vcs[0].name = 'wc'
        self.assertEqual(vcs.name, 'wc')
        vcs.type = 'PR'
        self.assertEqual(vcs.type, 'PR')
        vcs.base_branch = 'base'
        self.assertEqual(vcs.base_branch, 'base')
        vcs.branch = 'master'
        self.assertEqual(vcs.branch, 'master')
        vcs.base_commit = '123'
        self.assertEqual(vcs.base_commit, '123')
        vcs.clean_wa = True
        self.assertTrue(vcs.clean_wa)
        vcs.pr_number = '123'
        self.assertEqual(vcs.pr_number, '123')
        vcs.commit_id = '123'
        self.assertEqual(vcs.commit_id, '123')
        vcs.system = 'git'
        self.assertEqual(vcs.system, 'git')
        vcs.url = 'url'
        self.assertEqual(vcs.url, 'url')

    def test_target(self):
        build = Build()
        with self.assertRaises(TypeError):
            build.target = 'invalid'
        target = Target()
        build.target = target
        target.operating_system = "win32"
        self.assertEqual(build.target.operating_system, 'win32')
        target.type = "hardware"
        self.assertEqual(build.target.type, 'hardware')
        with self.assertRaises(TypeError):
            target.hardware = 'invalid'
        hw = Hardware()
        target.hardware = hw
        hw.model = 'abc'
        self.assertEqual(build.target.hardware.model, 'abc')
        hw.vendor = 'def'
        self.assertEqual(build.target.hardware.vendor, 'def')
        hw.rev = 'qwe'
        self.assertEqual(build.target.hardware.rev, 'qwe')
        hw.meta = 'ooo'
        self.assertEqual(build.target.hardware.meta, 'ooo')




# pylint: disable=C0103

"""
OpenTMI module for Test Job
"""
from opentmi_client.utils.Base import BaseApi
from opentmi_client.utils.decorators import setter_rules


class Job(BaseApi):
    """
    Job Class
    """
    def __init__(self, id_=None):
        """
        Constructor for Job
        :param id_: String
        """
        super(Job, self).__init__()
        if id_:
            self.id = id_

    @property
    def id(self):
        """
        Getter for job id
        :return: String
        """
        return self.get("id")

    @id.setter
    @setter_rules()
    def id(self, value):
        """
        Setter for Job id
        :param value: String
        """
        self.set("id", value)



# pylint: disable=missing-docstring

import json
import unittest
from mock import patch
from requests import Response, RequestException
from opentmi_client.transport import Transport
from opentmi_client.utils.exceptions import TransportException

# This method will be used by the mock to replace requests.get, post, put
def mocked_requests(*args, **kwargs):
    class MockResponse(Response):
        def __init__(self, json_data, status_code):
            Response.__init__(self)
            self.json_data = json_data
            self.status_code = status_code

        @property
        def text(self):
            return json.dumps(self.json_data)

        def json(self):
            return self.json_data

    if args[0] == 'RequestException':
        raise RequestException()
    elif args[0] == 'ValueError':
        raise ValueError("")
    elif args[0] == 'TypeError':
        raise TypeError("")
    elif args[0] == 'NotFound' or kwargs.get('NotFound'):
        raise MockResponse(None, 404)
    elif args[0] == 'status_300':
        return MockResponse(None, 300)

    return MockResponse({}, 200)


class TestRequest(unittest.TestCase):
    def test_is_success(self):
        resp = Response()
        resp.status_code = 200
        self.assertTrue(Transport.is_success(resp))
        resp.status_code = 299
        self.assertTrue(Transport.is_success(resp))
        resp.status_code = 300
        self.assertFalse(Transport.is_success(resp))
        resp.status_code = 199
        self.assertFalse(Transport.is_success(resp))

    def test_set_host(self):
        transport = Transport()
        HOST = "127.0.0.1"
        transport.set_host(HOST)
        self.assertEqual(transport.token, None)
        self.assertEqual(transport.host, "http://"+HOST)

        transport.set_host("http://a.b.c@"+HOST)
        self.assertEqual(transport.token, "a.b.c")
        self.assertEqual(transport.host, "http://"+HOST)
        self.assertTrue(transport.has_token())

        transport.set_host("https://aa.bb.cc@"+HOST)
        self.assertEqual(transport.token, "aa.bb.cc")
        self.assertEqual(transport.host, "https://"+HOST)

    def test_get_json_not_found(self):
        transport = Transport()
        with self.assertRaises(TransportException):
            transport.get_json("localhost")

    def test_get_post_not_found(self):
        transport = Transport()
        with self.assertRaises(TransportException):
            transport.post_json("localhost", {})

    def test_get_put_not_found(self):
        transport = Transport()
        with self.assertRaises(TransportException):
            transport.put_json("localhost", {})

    @patch('requests.get', side_effect=mocked_requests)
    def test_get_json(self, mock_post):
        transport = Transport()
        transport.set_token("mytoken")
        url = "https://localhost"
        self.assertIsInstance(transport.get_json(url), dict)

    @patch('requests.post', side_effect=mocked_requests)
    def test_post_json(self, mock_post):
        transport = Transport()
        url = "https://localhost"
        data = {}
        self.assertIsInstance(transport.post_json(url, data), dict)

    @patch('requests.put', side_effect=mocked_requests)
    def test_put_json(self, mock_put):
        transport = Transport()
        url = "https://localhost"
        data = {
            "title": "foo",
            "body": 'bar',
            "userId": 1
        }
        self.assertIsInstance(transport.put_json(url, data), dict)

    @patch('requests.get', side_effect=mocked_requests)
    def test_get_json_exceptions(self, mock_get):
        transport = Transport()
        with self.assertRaises(TransportException):
            transport.get_json("RequestException")
        with self.assertRaises(TransportException):
            transport.get_json("ValueError")
        with self.assertRaises(TransportException):
            transport.get_json("TypeError")
        with self.assertRaises(TransportException):
            transport.get_json("status_300")

    @patch('requests.post', side_effect=mocked_requests)
    def test_post_json_exceptions(self, mock_get):
        transport = Transport()
        with self.assertRaises(TransportException):
            transport.post_json("RequestException", {})
        with self.assertRaises(TransportException):
            transport.post_json("ValueError", {})
        with self.assertRaises(TransportException):
            transport.post_json("TypeError", {})
        with self.assertRaises(TransportException):
            transport.post_json("status_300", {})

    @patch('requests.put', side_effect=mocked_requests)
    def test_put_json_exceptions(self, mock_get):
        transport = Transport()
        with self.assertRaises(TransportException):
            transport.put_json("RequestException", {})
        with self.assertRaises(TransportException):
            transport.put_json("ValueError", {})
        with self.assertRaises(TransportException):
            transport.put_json("TypeError", {})
        with self.assertRaises(TransportException):
            transport.put_json("status_300", {})



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
:copyright: (c) 2020 by Jussi Vatjus-Anttila
:license: MIT, see LICENSE for more details.
"""
import os
import sys
from distutils.core import setup
from setuptools import find_packages
from setuptools.command.install import install

DESCRIPTION = "opentmi-client"
OWNER_NAMES = 'Jussi Vatjus-Anttila'
OWNER_EMAILS = 'jussiva@gmail.com'
VERSION = '0.8.0'


# Utility function to cat in a file (used for the README)
def read(fname):
    return open(os.path.join(os.path.dirname(__file__), fname)).read()


class VerifyVersionCommand(install):
    """Custom command to verify that the git tag matches our version"""
    description = 'verify that the git tag matches our version'

    def run(self):
        tag = os.getenv('CIRCLE_TAG')
        version = "v" + VERSION
        if tag != version:
            info = "Git tag: {0} does not match the" \
                   "version of this app: {1}".format(tag, version)
            sys.exit(info)


setup(name='opentmi_client',
      version=VERSION,
      description=DESCRIPTION,
      long_description=read('README.md'),
      long_description_content_type='text/markdown',
      author=OWNER_NAMES,
      author_email=OWNER_EMAILS,
      maintainer=OWNER_NAMES,
      maintainer_email=OWNER_EMAILS,
      url='https://github.com/OpenTMI/opentmi-pyclient.git',
      packages=find_packages(exclude=['test', 'log', 'htmlcov', 'docs']),
      include_package_data=True,
      license="MIT",
      tests_require=[
          "coverage",
          "mock",
          "pylint==2.5.2",
          "Sphinx",
          "nose"
      ],
      test_suite='test',
      keywords='opentmi ci cd api sdk',
      entry_points={
          "console_scripts": [
              "opentmi=opentmi_client:opentmiclient_main",
          ]
      },
      install_requires=[
          "requests>=2.13.0",
          "jsonmerge",
          "six",
          "pydash",
          "junitparser<2.0.0"
      ],
      classifiers=[
          "Programming Language :: Python :: 3.6",
          "Programming Language :: Python :: 3.7",
          "License :: OSI Approved :: MIT License",
          "Operating System :: OS Independent",
      ],
      cmdclass={
          'verify': VerifyVersionCommand,
      })



# pylint: disable=missing-docstring

import os
import unittest
from mock import patch
from opentmi_client.utils import is_object_id, resolve_host, resolve_token, archive_files, remove_empty_from_dict, requires_logged_in


class Test:
    def __init__(self, logged_in):
        self.logged_in = logged_in

    @requires_logged_in
    def api(self):
        return True

    def try_login(self, raise_if_fail):
        return self

    @property
    def is_logged_in(self):
        return self.logged_in

class TestTools(unittest.TestCase):
    def test_is_object_id(self):
        self.assertEqual(is_object_id(""), False)
        self.assertEqual(is_object_id("asd"), False)
        self.assertEqual(is_object_id(None), False)
        self.assertEqual(is_object_id(1), False)
        self.assertEqual(is_object_id("1234567890abcdef78901234"), True)

    def test_resolve_host(self):
        self.assertEqual(resolve_host("1.2.3.4"), "http://1.2.3.4")
        self.assertEqual(resolve_host("1.2.3.4", 80), "http://1.2.3.4")
        self.assertEqual(resolve_host("1.2.3.4", 8000), "http://1.2.3.4:8000")
        self.assertEqual(resolve_host("1.2.3.4:3000"), "http://1.2.3.4:3000")
        self.assertEqual(resolve_host("http://1.2.3.4"), "http://1.2.3.4")
        self.assertEqual(resolve_host("https://1.2.3.4"), "https://1.2.3.4")
        self.assertEqual(resolve_host("https://mydomain"), "https://mydomain")
        self.assertEqual(resolve_host("https://1.2.3.4:3000"), "https://1.2.3.4:3000")
        self.assertEqual(resolve_host("https://1.2.3.4", 3000), "https://1.2.3.4:3000")
        self.assertEqual(resolve_host("https://1.2.3.4", 3000), "https://1.2.3.4:3000")

    def test_resolve_token(self):
        self.assertEqual(resolve_token("http://1.2.3.4"), None)
        self.assertEqual(resolve_token("http://a.b.c@1.2.3.4"), "a.b.c")
        self.assertEqual(resolve_token("https://aa.bb.cc@1.2.3.4"), "aa.bb.cc")

    def test_archive(self):
        zip = 'temp.zip'
        cur_dir = os.path.dirname(__file__)
        file_to_zip = os.path.join(cur_dir, 'test_tools.py')
        archive_files([file_to_zip], zip, cur_dir)
        self.assertTrue(os.path.exists(zip))
        os.remove(zip)

    def test_requires_logged_in(self):
        test = Test(logged_in=True)
        with patch.object(test, 'try_login') as monkey:
            self.assertTrue(test.api())
            self.assertTrue(monkey.notCalled())

        test = Test(logged_in=False)
        with patch.object(test, 'try_login') as monkey:
            self.assertTrue(test.api())
            self.assertTrue(monkey.calledOnceWith(raise_if_fail=True))

    def test_remove_empty_from_dict(self):
        data = {"a": {"b": {}, "c": [], "d": 1}}
        self.assertEqual(remove_empty_from_dict(data), {"a": {"d": 1}})

if __name__ == '__main__':
    unittest.main()



""" OpenTMI module """
# Internal imports
from opentmi_client.utils.Base import BaseApi
from opentmi_client.utils.decorators import setter_rules


class Skip(BaseApi):
    """ Skip class """

    @property
    def value(self):
        """
        Getter for skip value
        :return: bool
        """
        return self.get("value")

    @value.setter
    @setter_rules(value_type=bool)
    def value(self, value: bool):
        """
        Setter for test case skip value
        :param value: bool
        """
        self.set("value", value)

    @property
    def reason(self):
        """
        Getter for skip reason
        :return: str
        """
        return self.get("reason")

    @reason.setter
    @setter_rules()
    def reason(self, value: str):
        """
        Setter for test case skip reason
        :param value: bool
        """
        self.set("reason", value)



"""
OpenTMI module for Target
"""
from opentmi_client.utils.Base import BaseApi
from opentmi_client.utils.decorators import setter_rules
from opentmi_client.api.build.Hardware import Hardware


class Target(BaseApi):
    """
    Target Class
    """
    def __init__(self):
        super(Target, self).__init__()
        self.hardware = Hardware()

    @property
    def type(self):
        """
        Getter for target type
        :return: String
        """
        return self.get("type")

    @type.setter
    @setter_rules(value_type=str, enum="simulate hardware")
    def type(self, value):
        """
        Setter for target type
        :param value: String (simulate/hardware)
        :return: value
        """
        return self.set("type", value)

    @property
    def operating_system(self):
        """
        Getter for target type
        :return: String
        """
        return self.get("os")

    @operating_system.setter
    @setter_rules(value_type=str, enum="win32 win64 unix32 unix64 mbedOS unknown")
    def operating_system(self, value):
        """
        Setter for target os
        :param value: String (win32 win64 unix32 unix64 mbedOS unknown)
        :return: value
        """
        return self.set("os", value)

    @property
    def hardware(self):
        """
        Getter for target hw
        :return: Hardware
        """
        return self.get("hw")

    @hardware.setter
    @setter_rules(value_type=Hardware)
    def hardware(self, value):
        """
        Setter for target hw
        :param value: Hardware
        :return: value
        """
        return self.set("hw", value)



"""
OpenTMI module for Resource.status
"""
from opentmi_client.utils.Base import BaseApi
from opentmi_client.utils.decorators import setter_rules


class Status(BaseApi):
    """
    Status Class
    """

    @property
    def value(self):
        """
        Getter for value
        :return: String
        """
        return self.get("value")

    @value.setter
    @setter_rules(enum=[
        'active',
        'maintenance',
        'storage',
        'broken'
    ])
    def value(self, value):
        """
        Setter for value
        :param value: string
        """
        self.set("value", value)

    @property
    def note(self):
        """
        Getter for note
        :return: String
        """
        return self.get("note")

    @note.setter
    @setter_rules()
    def note(self, value):
        """
        Setter for note
        :param value: string
        """
        self.set("note", value)

    @property
    def availability(self):
        """
        Getter for availability
        :return: str
        """
        return self.get("availability")

    @availability.setter
    @setter_rules(enum=['free', 'reserved'])
    def availability(self, value):
        """
        Setter for availability
        :param value: string
        """
        self.set("availability", value)



import unittest
from opentmi_client.api.testcase.Testcase import Testcase
from opentmi_client.api.testcase.Compatible import Compatible
from opentmi_client.api.testcase.Status import Status
from opentmi_client.api.testcase.Execution import Execution
from opentmi_client.api.testcase.OtherInfo import OtherInfo


class TestTestcase(unittest.TestCase):
    def test_construct(self):
        test = Testcase(tcid="")
        self.assertIsInstance(test, Testcase)
        self.assertEqual(test.data, {})

    def test_str(self):
        test = Testcase(tcid="id")
        self.assertEqual(str(test), "id")

    def test_properties(self):
        test = Testcase(tcid="abc")
        self.assertEqual(test.tcid, "abc")
        self.assertIsInstance(test.execution, Execution)
        self.assertIsInstance(test.other_info, OtherInfo)
        self.assertIsInstance(test.status, Status)
        self.assertIsInstance(test.compatible, Compatible)
        test.execution.skip.value = True
        self.assertTrue(test.execution.skip.value)
        test.execution.skip.reason = "note"
        self.assertEqual(test.execution.skip.reason, "note")
        test.other_info.title = "aa"
        self.assertEqual(test.other_info.title, "aa")
        test.other_info.description = "desc"
        self.assertEqual(test.other_info.description, "desc")
        test.other_info.type = "installation"
        self.assertEqual(test.other_info.type, "installation")
        test.other_info.keywords = ["aa"]
        self.assertEqual(test.other_info.keywords, ["aa"])
        test.status.value = "released"
        self.assertEqual(test.status.value, "released")
        test.status.verification.value = True
        self.assertTrue(test.status.verification.value)
        self.assertDictEqual(test.data, {
            "tcid": "abc",
            "execution": {
                "skip": {
                    "value": True,
                    "reason": "note"
                }
            },
            "status": {
                "value": "released",
                "verification": {
                    "value": True
                }
            },
            "other_info": {
                "title": "aa",
                "description": "desc",
                "type": "installation",
                "keywords": ["aa"]
            }
        })
